-- Version info
local version = 0.1
logit = true
function mylog(text,override)
  local over
  if (override) then over = override end
  if over ~= true then over = false end
  if logit or over then
    log("Roles: " .. text)
  end
end

-- Need the default utilities
require("util")
-- Need the roles definition and presets--
require("roles")
-- Need the scripts for launching the rocket, which is part of this scenario
local silo_script = require("silo-script")

-- Need the mod-gui library from game's lualib
local mod_gui = require("mod-gui")

--[[
    Event Handlers
  ]]
  
-- Start of the scenario
script.on_init(function()
  mylog("on_init")
  global.version = version
  load_roles()
  -- put it in a file
  game.write_file('permissions.lua', serpent.block(global.rolespermissions))
  game.write_file('defines.lua', serpent.block(defines))

  -- Create a 1x1 Lobby for players to be placed 
  local surface = game.create_surface("Lobby",{width = 1, height = 1})
  surface.set_tiles({{name = "out-of-map",position = {1,1}}})
  for k, entity in pairs (surface.find_entities()) do
    entity.destroy()
  end
  surface.destroy_decoratives({{-500,-500},{500,500}})
 -- Freeplay init_script
  silo_script.on_init()
end)

-- When a player joins for the first time

script.on_event(defines.events.on_player_created, function(event)
  --[[ 
    If this is the first player, don't do anything special
    Otherwise, set them an initial role?
  ]]
  mylog("on_player_created")
  local player = game.players[event.player_index]
  if event.player_index == 1 then
    player.insert{name = "iron-plate", count = 8}
    player.insert{name = "burner-mining-drill", count = 1}
    player.insert{name = "stone-furnace", count = 1}
    player.insert{name = "pistol", count = 1}
    player.insert{name = "firearm-magazine", count = 10}
    player.insert{name = "car", count = 1}
    player.insert{name = "tank", count = 1}
    player.insert{name = "power-armor-mk2", count = 1}
    player.insert{name = "fusion-reactor-equipment", count = 2}
    player.insert{name = "night-vision-equipment", count = 1}
    player.insert{name = "coal", count = 52*50}
    return
  end
--[[ Not necessary, done in player joined?
  if not player.admin then
    local character = player.character
    player.character = nil
      if character then
      character.destroy()
    end
  end
  ]]
  
  -- Show the player the minimap
  player.force.chart(player.surface, {{player.position.x - 200, player.position.y - 200}, {player.position.x + 200, player.position.y + 200}})
  if (#game.players <= 1) then
    game.show_message_dialog{text = {"msg-introduction"}}
  else
    player.print({"msg-introduction"})
  end

  --Freeplay player_created script
  silo_script.on_player_created(event)
end)

-- When a player (rejoins) [event also called the first time they join]
script.on_event(defines.events.on_player_joined_game, function(event)
  mylog("on_player_joined_game")
  -- If this is the game creator (temporary)
  if event.player_index == 1 then return end
  local player = game.players[event.player_index]
  -- Return immediately if they've previously joined
  if not (player and player.valid) then return end
  -- if player.admin then return end --Admins are immediately placed in the game
  if player.force.name ~= "player" then return end --If they are not on the player force, they have already have a role
  local character = player.character
  player.character = nil
  if character then character.destroy() end
  -- player.set_controller{type = defines.controllers.ghost}
  player.teleport({0,1000}, game.surfaces.Lobby)
  create_roles_frame(player)
end)

-- When a player puts an item
script.on_event(defines.events.on_put_item,function(event)
  mylog("on_put_item")
  -- may not need it
end)

-- When a player builds an entity
script.on_event(defines.events.on_built_entity,function(event)
  -- test here if player can build this and if not, pick it up again
  local player = game.players[event.player_index]
  local entity = event.created_entity
  local localised_name
  local entity_name
  if entity.name == "entity-ghost" then
    localised_name = entity.ghost_localised_name
    entity_name = entity.ghost_name
  else
    localised_name = entity.localised_name
    entity_name = entity.name
  end
  mylog("on_built_entity: " .. entity_name)
  if entity_name == "stone-furnace"  then
    player.print({"msg-cannot build",localised_name})
    -- Return item to inventory if it isn't a ghost
    if entity.name ~= "entity-ghost" then
      player.insert{name = "stone-furnace", count = 1}
    end
    entity.destroy()
  end
end)

-- When a player changes the check state of GUI Element
script.on_event(defines.events.on_gui_checked_state_changed,function(event)
  mylog("on_gui_checked_state_changed: " .. event.element.parent.name)
  mylog("on_gui_checked_state_changed: " .. event.element.name)
  if event.element.parent.name == "roles_frame" then
    local elm = event.element
    if elm.type == "radiobutton" then
      if elm.state == false then --[[can't deactivate already active radio button]]
        elm.state = true
      else
        for _,child in pairs(elm.parent.children) do
          mylog("on_gui_checked_state_changed: " .. child.name)
          if child.type == "radiobutton" then
            child.state = false
          end
        end
        elm.state = true --[[turn it back on]]
      end
    end
  end
end)

-- When a player clicks a selectable GUI Element
script.on_event(defines.events.on_gui_click,function(event)
  mylog("on_gui_click: " .. event.element.parent.name)
  if event.element.parent.name == "confirm_role_button_flow" then
    -- Player has asked to spectate, claimed to be admin, or asked for a single role
    mylog("on_gui_click: Button: " .. event.element.name)
    local player = game.players[event.player_index]
    local GUI = event.element.parent
    GUI = GUI.parent
    if event.element.name == "button-roles-spectate" then
        -- Player wants to spectate
          local spectator = game.forces["spectator"]
          if not (spectator and spectator.valid) then
            spectator = game.create_force("spectator")
          end
        -- Assign permissions group that disables the start_walking permission
        local permission = game.permissions.create_group("spectator")
        permission.set_allows_action(defines.input_action.start_walking,false)
        permission.add_player(player)
        --player.disable_recipe_groups()
        --player.disable_recipe_subgroups()
        -- player.force = "spectator"
        --[[local recipes = table.deepcopy(spectator.recipes)
        local recipeStr = ""
        for _,v in pairs(recipes) do
          recipeStr = recipeStr .. v.name .. " : " .. spectator.recipes[v.name].group.name .. " :" .. spectator.recipes[v.name].subgroup.name .. "\n"
          --recipeStr = recipeStr .. serpent.block(spectator.recipes[v.name])
        end
        game.write_file('recipes.lua', recipeStr) ]]
        player.teleport({0,0}, "nauvis")
        player.print(player.name .. " has joined as a Spectator")
    elseif event.element.name == "button-roles-all" then
        -- Player claims to be admin
        player.print("Admin: " .. player.name .. " has joined")
    else
        -- Player is asking for single role
        player.print(player.name .. " has requested a Role")
    end
    -- Close the dialog
    GUI.destroy()
  end
  silo_script.on_gui_click(event)
end)

-- When a player's gun inventory changes
--[[
Needs test for whether player can be armed
]]
script.on_event(defines.events.on_player_gun_inventory_changed,function(event)
  mylog("on_player_gun_inventory_changed: " .. event.player_index)
  local player = game.players[event.player_index]
  local armor = player.get_inventory(defines.inventory.player_armor)
  mylog("armor: " .. serpent.block(armor.get_contents()))
  player.character.damage(5000,"enemy")
  -- armor.entity_owner.damage(5000,"enemy")
  local inventory = player.get_inventory(defines.inventory.player_guns)
  if (inventory) then
    if not inventory.is_empty() then
      player.print({"msg-noarmsrights"})
      local guns = inventory.get_contents()
      local gun = {}
      local main_inventory = player.get_inventory(defines.inventory.player_main)
      mylog(serpent.block(guns))
      -- Move gun back into main inventory or spill on ground
      for k,v in pairs(guns) do
        gun.name = k
        gun.count = v
        mylog(serpent.block(gun))
        if main_inventory.insert(gun) == 0 then
          local surface = player.surface
          surface.spill_item_stack(player.position,gun,false)
        end
      end
      -- Clear gun inventory
      inventory.clear()
    end
  end

end)

-- When a player's ammo inventory changes
--[[
Needs test for whether player can be armed
]]
script.on_event(defines.events.on_player_ammo_inventory_changed,function(event)
  mylog("on_player_ammo_inventory_changed: " .. event.player_index)
  local player = game.players[event.player_index]
  local ammotype = {}
  --[[
    TODO: Need to figure out how to sense when ammo is put in a car/tank/turret
    or decide whether to allow it (Define 'bear arms')
  ]]
  ammotype[1] = defines.inventory.player_ammo
  --ammotype[2] = defines.inventory.car_ammo
  --ammotype[3] = defines.inventory.turret_ammo
  mylog(serpent.block(ammotype))
  for k = 1,1 do -- needs to be 3 when TODO above is fixed?
    local inventory = player.get_inventory(ammotype[k])
    mylog("inventory : " .. ammotype[k])
    if (inventory) then
      if not inventory.is_empty() then
        player.print({"msg-noammorights"})
        local ammo = inventory.get_contents()
        local shells = {}
        local main_inventory = player.get_inventory(defines.inventory.player_main)
        mylog("ammotype : " .. serpent.block(ammo))
        -- Move ammo back into main inventory or spill on ground
        for k,v in pairs(ammo) do
          shells.name = k
          shells.count = v
          mylog("shells : " .. serpent.block(shells))
          if main_inventory.insert(shells) == 0 then
            local surface = player.surface
            surface.spill_item_stack(player.position,shells,false)
          end
        end
        -- Clear ammo inventory
        inventory.clear()
      end
    end
  end
end)

-- Every game tick
script.on_event(defines.events.on_tick,function(event)
end)

--[[ functions called in the events
]]

-- Creates and display the roles frame
function create_roles_frame(player)
  local frame = mod_gui.get_frame_flow(player).add{name = "roles_frame", type = "frame", direction = "vertical", caption={"msg-ask-roles"}}
  --[[ Need to add some tooltips. This code does not work
  frame.add{type = "label", name = "radio-roles-defender-tooltip", tooltip = {"radio-roles-defender-tooltip"}}
  ]]
  local roles = load_roles(true)
  for k,v in pairs(roles.rolenames) do 
    local btn = frame.add{type='radiobutton', name="radio-roles-"..v, caption={"radio-roles-"..v},state = false}
  end
  local button_flow = frame.add{type = "flow", direction = "horizontal", name = "confirm_role_button_flow"}
  button_flow.add{type = "button", name = "button-roles-spectate", caption = {"button-roles-spectate"}}
  button_flow.add{type = "button", name = "button-roles-all", caption = {"button-roles-all"}}
  button_flow.add{type = "button", name = "button-roles-thisone", caption = {"button-roles-thisone"}}
end